#Storyboard React Native Plugin

#Installation
`npm install --save [Path To Storyboard React plugin folder]` or `yarn add file:[Path To Storyboard React plugin folder]`

`react-native link react-native-storyboard`

On upgrade please uninstall the previous plugin.

#Uninstall
`react-native unlink react-native-storyboard`
`npm uninstall --save react-native-storyboard` or `yarn remove react-native-storyboard` 

#Run
    const { Storyboard, StoryboardTouchMode} = require('react-native-storyboard');

-   Call `Storyboard.start(reportUrl, appId);`
        
-   Call `Storyboard.startWith(options);`
    
    where options is an object which may contain reportUrl and appId and\or firebase.
    
    For example:
    
    `var options = {reportUrl: "http://...", appId: "AAAA...", firebase: fire...};`


#API

### Custom Event
Custom event can be reported using the following API :
`Storyboard.reportEvent("eventName", {param : "paramValue"});`

### Sensitive data
To set a view as sensitive call Storyboard.setViewAsSensitive using the view ref.
For example :

`var optionalVisibilityFlags = {
     maskOnAlpha: false,
     maskOnHidden: true,
     maskOnAllControllers: false
}`     
`\<Text ref={ v => Storyboard.setViewAsSensitive(v, optionalVisibilityFlags) }>\`
Note, please make sure to call the storyboard start API prior to sensitive view load time.

Optional parameter visibilityFlags controls masking in the following way :
##### maskOnAlpha : controls whether a sensitive view should be masked even if it's alpha property is set to 0. default is false.
##### maskOnHidden : controls whether a sensitive view should be masked even if it's hidden property is set to true. default is false.
##### maskOnAllControllers : controls whether a sensitive view should be masked even if covered by another view or UIViewController default is true.
##### touchMode : controls how user touches should be simulated in a sensitive view scope options are StoryboardTouchMode.NONE | StoryboardTouchMode.DISPLAY | StoryboardTouchMode.CENTER
### Tagging
To set a tag for a view call Storyboard.setViewTag using the view ref.
For example : '\<Button ref={ v => Storyboard.setViewTag(v, "MyCustomTag") }\'
Note, please make sure to call the storyboard start API prior to the view load time.


### Manually Track Screen
Configure custom screen segmentation using startScreen api.
`Storyboard.startScreen(screenName)`
`Storyboard.endScreen()`

Note: End screen api is optional and used to disable previous screen which was set using the startScreen api

### Hybrid mode  - Manually track WebView's
Enables hybrid replay - Web + Mobile events in the same session.
This api is useful for apps that are using webview
`Storyboard.trackView(webViewRef)`

##Example:

`<WebView
      ref={ref => (Storyboard.trackView(findNodeHandle(ref)))}
      source= { Platform.OS == 'ios' ? require('./assets/loginPage.html') : { uri: "file:///android_asset/loginPage.html" } }
      javaScriptEnabled = {true}
      domStorageEnabled = {true}
    />`

### React Navigation Supprt - track new loaded screens via react-navigation 
Enables automatic screen detection based on react navigation events.
This api should be invoked using the AppContainer object returned by createAppContainer() call.
`Storyboard.trackNavigation(AppContainer)`

##Example:

`const AppContainer = createAppContainer(SwitchNavigator);
Storyboard.trackNavigation(AppContainer);`
